#pragma once

#include "FBullCowGame.h"
#include <map>
#include <vector> 
#include <cstdlib>
#include <ctime>

// to make syntax Unreal friendly
#define Tmap std::map 
using int32 = int;


FBullCowGame::FBullCowGame() { Reset(); } // default constructor...when we create a new instance it tells it to use these values at creation i.e. defines the starting state. In these case it refers to the Reset method

// "your getters"
int32 FBullCowGame::GetCurrentTry() const { return MyCurrentTry; }
int32 FBullCowGame::GetHiddenWordLength() const { return MyHiddenWord.length(); }
bool FBullCowGame::IsGameWon() const { return bGameIsWon; }

int32 FBullCowGame::GetMaxTries() const 
{ 
	Tmap<int32, int32> WordLengthToMaxTries{ {3,4}, {4,7}, {5,10}, {6,16}, {7,20} };
	return WordLengthToMaxTries[MyHiddenWord.length()];
}

// followed by your methods
void FBullCowGame::Reset() 
{ 
	std::vector<FString> HiddenWordList{ "sad","made","speak","planet","rescind" };
	srand(static_cast<unsigned int>(time(0)));
	int32 WordListNumber = rand();
	int32 ChosenWord = (WordListNumber % 5);

	const FString HIDDEN_WORD = HiddenWordList[ChosenWord];  // this must be an isogram 
	MyHiddenWord = HIDDEN_WORD;
	MyCurrentTry = 1;
	bGameIsWon = false;
	return;
}

EGuessStatus FBullCowGame::CheckGuessValidity(FString Guess) const
{ 
	if (!IsIsogram(Guess))
	{
		return EGuessStatus::Not_Isogram;
	}
	else if (!IsLowercase(Guess))                                                                     
	{
		return EGuessStatus::Not_Lowercase;
	}
	else if (Guess.length() != GetHiddenWordLength()) 
	{
		return EGuessStatus::Wrong_Length;
	}
	else
	{
		return EGuessStatus::OK; 
	}
}


// receives a valid guess, increments turn, and returns count
FBullCowCount FBullCowGame::SubmitValidGuess(FString Guess)            
{
	MyCurrentTry++;
	FBullCowCount BullCowCount;
	int32 WordLength = MyHiddenWord.length(); // assuming same length as guess
	
	// loop through all the letters in hidden word
	for (int32 MHWChar = 0; MHWChar < WordLength; MHWChar++) {
		// compare letters against the guess
		for (int32 GChar = 0; GChar < WordLength; GChar++) {
			// if they match then	
			if (Guess[GChar] == MyHiddenWord[MHWChar])
			{
				if (MHWChar == GChar) 	// if they're in the same place			
				{
					BullCowCount.Bulls++;
				}
				else
				{
					BullCowCount.Cows++;
				}
			}
		}
	}
	if (BullCowCount.Bulls == WordLength)
	{
		bGameIsWon = true;
	}
	else
	{
		bGameIsWon = false;
	}
	return BullCowCount;
}

bool FBullCowGame::IsIsogram(FString Word) const 
{
	// treat 0 and 1 letter words as isograms
	if (Word.length() <= 1) { return true; }

	Tmap<char, bool> LetterSeen; // setup our map
	for (auto Letter : Word)	// for all letters of the word
	{
		Letter = tolower(Letter); // handle mixed case
		if (LetterSeen[Letter]) { // if the letter is in the map       // our if statement won't find the letter at first so it will default to false, which means we skip the IF statement (because not true) and go to the ELSE statement which makes all chars in the map true. If a letter repeats itself then we see we already have true value and the IF statement returns false i.e. we do not have an isogram
		return false; // we do NOT have an isogram at this stage
		} 
		else // otherwise
		{
			LetterSeen[Letter] = true; // add letter to the map as seen
		}
				
	}
	return true;
}

bool FBullCowGame::IsLowercase(FString Word) const
{	
	for (auto Letter : Word)
	{
		if (!islower(Letter))
		{
			return false;
		}
	}
	return true;
}
 