/* this is the console executable, that makes use of the BullCow class 
This acts as the view in the MVC pattern, and is responsible for all
user interaction. for game logic see the FBullCowGame class.
*/

#pragma once 

#include <iostream>
#include <string>
#include "FBullCowGame.h" 

// to make syntax Unreal friendly, make sure to check unreal engine's standard for type capitalization
using FText = std::string;  
using int32 = int;			

// function prototypes as outside a class
int main();
void PrintIntro();
void PlayGame();
FText GetValidGuess();
bool AskToPlayAgain();
void PrintGameSummary();


FBullCowGame BCGame; // instantiate a new game, which we re-use across plays
int main()
{
	bool bPlayAgain = false; 
	do
	{
		PlayGame();
		bPlayAgain = AskToPlayAgain();
	}
	while (bPlayAgain);
	
	return 0; // exit the application
}


void PrintIntro() 
{	
	std::cout << "Welcome to Bulls and Cows, a fun word game.\n";
	std::cout << "Can you think of the " << BCGame.GetHiddenWordLength();
	std::cout << " letter isogram I'm thinking of?\n";
	std::cout << std::endl;
	return;
}


// plays a single game to completion
void PlayGame()
{	
	BCGame.Reset();
	PrintIntro();
	int32 MaxTries = BCGame.GetMaxTries();

	// loop asking for guesses
	while ((!BCGame.IsGameWon()) && (BCGame.GetCurrentTry() <= MaxTries))
	{
		FText Guess = GetValidGuess(); 

		// submit valid guess to the game, and receive counts
		FBullCowCount BullCowCount = BCGame.SubmitValidGuess(Guess);  

		std::cout << "Bulls = " << BullCowCount.Bulls;
		std::cout << ". Cows = " << BullCowCount.Cows << "\n\n";
	}
	PrintGameSummary();
	return;
}

// loop continually until the user gives a valid guess
FText GetValidGuess() 
{	
	FText Guess = "";
	EGuessStatus Status = EGuessStatus::Invalid_Status;
	do {
		// get a guess from the player
		int32 CurrentTry = BCGame.GetCurrentTry();
		std::cout << "Try " << CurrentTry << " of " << BCGame.GetMaxTries();
		std::cout << ". Enter your guess: ";
		std::getline(std::cin, Guess);

		// check status and give feedback
		Status = BCGame.CheckGuessValidity(Guess);
		switch (Status) {
		case EGuessStatus::Wrong_Length:
			std::cout << "Please enter a " << BCGame.GetHiddenWordLength() << " letter word.\n\n";
			break;
		case EGuessStatus::Not_Isogram:
			std::cout << "Please enter a word without repeating letters.\n\n";
			break;
		case EGuessStatus::Not_Lowercase:
			std::cout << "Please enter an all lowercase word.\n\n";
			break;
		default:
			break;
		}
	} while (Status != EGuessStatus::OK);
	return Guess;
}

// ask player if they want to play again
bool AskToPlayAgain()
{
	std::cout << "Do you want to play again with a random hidden word (y/n)? ";
	FText Response = "";
	std::getline(std::cin, Response);
	std::cout << std::endl << std::endl;

	return (Response[0] == 'y') || (Response[0] == 'Y');
}

void PrintGameSummary()
{
	if (BCGame.IsGameWon())
	{
		std::cout << "WELL DONE - YOU WIN!\n\n";
	}
	else
	{
		std::cout << "Better luck next time!\n\n";
	}
	return;
}
